@@ -4,16 +4,19 @@ Dashboard definition classes.
 This module provides several classes to define a Dashboard.
 """
 
-from powa.json import JSONizable
+from operator import attrgetter
+from powa.compat import classproperty, with_metaclass
 from powa.framework import AuthHandler
-from powa.compat import with_metaclass, classproperty
+from powa.json import JSONizable
 from powa.ui_modules import MenuEntry
 from tornado.web import URLSpec
-from operator import attrgetter
+
 try:
     from collections import OrderedDict
-except:
+except ModuleNotFoundError:
     from ordereddict import OrderedDict
+
+import psycopg2
 from inspect import isfunction
 
 GLOBAL_COUNTER = 0
@@ -32,40 +35,145 @@ class DashboardHandler(AuthHandler):
         # Compatibility for tornado < 3:
         if not hasattr(self, "path_args"):
             self.path_args = args
-        params = OrderedDict(zip(self.params,
-                                 args))
-        param_dashboard = self.dashboard().parameterized_json(self, **params)
-        param_datasource = []
-        for datasource in self.datasources:
-            # ugly hack to avoid calling the datasource twice per
-            # DashboardPage (once because it's declared in the datasources,
-            # used to automatically register the URLSpecs, and once by the
-            # javascript facility that will look for configuration changes)
-            if datasource.url_name.startswith("datasource_ConfigChanges"):
-                continue
 
-            value = datasource.parameterized_json(self, **params)
-            value['data_url'] = self.reverse_url(datasource.url_name,
-                                                 *args)
-            param_datasource.append(value)
+        params = OrderedDict(zip(self.params, args))
+        title = self.dashboard().title % params
 
-        # tell the frontend how to get the configuration changes, if the
-        # DashboardPage provided it
-        param_timeline = None
-        if (self.timeline):
-            param_timeline = self.reverse_url(self.timeline.url_name, *args)
+        if self.request.headers.get("Content-Type") == "application/json":
+            param_dashboard = self.dashboard().parameterized_json(
+                self, **params
+            )
+            param_datasource = []
+            for datasource in self.datasources:
+                # ugly hack to avoid calling the datasource twice per
+                # DashboardPage (once because it's declared in the datasources,
+                # used to automatically register the URLSpecs, and once by the
+                # javascript facility that will look for configuration changes)
+                if datasource.url_name.startswith("datasource_ConfigChanges"):
+                    continue
 
-        return self.render(self.template,
-                           dashboard=param_dashboard,
-                           datasources=param_datasource,
-                           timeline=param_timeline,
-                           title=self.dashboard().title % params)
+                value = datasource.parameterized_json(self, **params)
+                value["data_url"] = self.reverse_url(
+                    datasource.url_name, *args
+                )
+                param_datasource.append(value)
+
+            # tell the frontend how to get the configuration changes, if the
+            # DashboardPage provided it
+            param_timeline = None
+            if self.timeline:
+                # Dashboards can specify a subset of arguments to use for the
+                # timeline.
+                if self.timeline_params:
+                    tl_args = [
+                        params[prm]
+                        for prm in self.params
+                        if prm in self.timeline_params
+                    ]
+                else:
+                    tl_args = args
+                param_timeline = self.reverse_url(
+                    self.timeline.url_name, *tl_args
+                )
+
+            last = len(self.breadcrumb) - 1
+            breadcrumbs = [
+                {
+                    "text": item.title,
+                    "href": self.reverse_url(
+                        item.url_name, *item.url_params.values()
+                    ),
+                }
+                for i, item in enumerate(reversed(self.breadcrumb))
+            ] + [
+                {
+                    "text": item.children_title,
+                    "children": (
+                        [
+                            {
+                                "url": self.reverse_url(
+                                    child.url_name, *child.url_params.values()
+                                ),
+                                "title": child.title,
+                            }
+                            for child in item.children
+                        ]
+                        if item.children and i == last
+                        else None
+                    ),
+                }
+                for i, item in enumerate(reversed(self.breadcrumb))
+                if i == last and item.children
+            ]
+
+            return self.render_json(
+                dict(
+                    dashboard=param_dashboard,
+                    datasources=param_datasource,
+                    timeline=param_timeline,
+                    title=title,
+                    breadcrumbs=breadcrumbs,
+                    handler=self.to_json(),
+                )
+            )
+
+        return self.render(self.template, title=title)
 
     @property
     def database(self):
         params = dict(zip(self.params, self.path_args))
         return params.get("database", None)
 
+    @property
+    def server(self):
+        params = dict(zip(self.params, self.path_args))
+        return params.get("server", None)
+
+    @property
+    def notify_allowed(self):
+        """
+        Returns whether the current user is allowed to use the NOTIFY-based
+        communication with powa-collector
+        """
+        conn = self.connect()
+        cur = conn.cursor()
+
+        powa_roles = [
+            "powa_signal_backend",
+            "powa_write_all_data",
+            "powa_admin",
+        ]
+        roles_to_test = []
+
+        # pg 14+ introduced predefined roles
+        if conn.server_version >= 140000:
+            roles_to_test.append("pg_signal_backend")
+
+        try:
+            cur.execute(
+                """WITH s(v) AS (
+                    SELECT unnest(%s::text[])
+                    UNION ALL
+                    SELECT rolname
+                    FROM {powa}.powa_roles
+                    WHERE powa_role = ANY (%s)
+                )
+                SELECT bool_or(pg_has_role(current_user, v, 'USAGE'))
+                    FROM s""",
+                (roles_to_test, powa_roles),
+            )
+        except psycopg2.Error:
+            conn.rollback()
+            return False
+
+        row = cur.fetchone()
+
+        # should not happen
+        if not row:
+            return False
+
+        return row[0] is True
+
     @property
     def breadcrumb(self):
         params = OrderedDict(zip(self.params, self.path_args))
@@ -84,19 +192,29 @@ class MetricGroupHandler(AuthHandler):
 
     def get(self, *params):
         url_params = dict(zip(self.params, params))
-        url_query_params = dict((
-            (key, value[0].decode('utf8'))
-            for key, value
-            in self.request.arguments.items()))
+        url_query_params = dict(
+            (
+                (key, value[0].decode("utf8"))
+                for key, value in self.request.arguments.items()
+            )
+        )
         url_params.update(url_query_params)
         url_params = self.add_params(url_params)
-        query = self.query
-        if (query is not None):
-            values = self.execute(query, params=url_params)
-            data = {"data": [self.process(val, **url_params)
-                             for val in values]}
+        res = self.query
+        if res is None:
+            query = None
+        elif isinstance(res, str):
+            query = res
         else:
-            data = {}
+            query = res[0]
+            url_params.update(res[1])
+        data = {"data": []}
+        if query is not None:
+            values = self.execute(query, params=url_params)
+            if values is not None:
+                data = {
+                    "data": [self.process(val, **url_params) for val in values]
+                }
 
         data = self.post_process(data, **url_params)
         self.render_json(data)
@@ -111,7 +229,7 @@ class MetricGroupHandler(AuthHandler):
         Arguments:
             handler (tornado.web.RequestHandler):
                 the current request handler
-            val (sqlalchemy.engine.result.RowProxy):
+            val (psycopg2.RealDictRow):
                 the row to process
             kwargs (dict):
                 the current url_parameters
@@ -128,7 +246,7 @@ class MetricGroupHandler(AuthHandler):
         Arguments:
             handler (tornado.web.RequestHandler):
                 the current request handler
-            val (sqlalchemy.engine.result.RowProxy):
+            val (psycopg2.RealDictRow):
                 the row to process
             kwargs (dict):
                 the current url_parameters
@@ -147,6 +265,7 @@ class DataSource(JSONizable):
             a subclass of RequestHandler used to process this DataSource.
 
     """
+
     datasource_handler_cls = None
     data_url = None
     enabled = True
@@ -158,13 +277,11 @@ class DataSource(JSONizable):
         """
         return "datasource_%s" % cls.__name__
 
-
     @classproperty
     def parameterized_json(cls, handler, **parms):
         return cls.to_json()
 
 
-
 class Metric(JSONizable):
     """
     An indivudal Metric.
@@ -228,7 +345,8 @@ class Dashboard(JSONizable):
             if (12 % len(row)) != 0:
                 raise ValueError(
                     "Each widget row length must be a "
-                    "divisor of 12 (have: %d)" % len(row))
+                    "divisor of 12 (have: %d)" % len(row)
+                )
 
     @property
     def widgets(self):
@@ -247,9 +365,11 @@ class Dashboard(JSONizable):
 
     def to_json(self):
         self._validate_layout()
-        return {'title': self.title,
-                'type': 'dashboard',
-                'widgets': self.widgets}
+        return {
+            "title": self.title,
+            "type": "dashboard",
+            "widgets": self.widgets,
+        }
 
     def param_widgets(self, _, **params):
         param_rows = []
@@ -261,44 +381,46 @@ class Dashboard(JSONizable):
         return param_rows
 
     def parameterized_json(self, _, **params):
-        return {'title': self.title % params,
-                'type': 'dashboard',
-                'widgets': self.param_widgets(_, **params)}
+        return {
+            "title": self.title % params,
+            "type": "dashboard",
+            "widgets": self.param_widgets(_, **params),
+        }
+
 
 class Panel(JSONizable):
-
     def __init__(self, title, widget):
         self.title = title
         self.widget = widget
 
     def to_json(self):
-        return {"title": self.title,
-                "widget": self.widget}
+        return {"title": self.title, "widget": self.widget}
 
     def parameterized_json(self, _, **args):
-        return {"title": self.title % args,
-                "type": "panel",
-                "widget": self.widget.parameterized_json(_, **args)}
+        return {
+            "title": self.title % args,
+            "type": "panel",
+            "widget": self.widget.parameterized_json(_, **args),
+        }
 
 
 class TabContainer(JSONizable):
-
     def __init__(self, title, tabs=None):
         self.title = title
         self.tabs = tabs or []
 
     def to_json(self):
-        tabs = []
-        return {'title': self.title,
-                'tabs': self.tabs}
+        return {"title": self.title, "tabs": self.tabs}
 
     def parameterized_json(self, _, **params):
         tabs = []
         for tab in self.tabs:
             tabs.append(tab.parameterized_json(_, **params))
-        return {'title': self.title % params,
-                'type': 'tabcontainer',
-                'tabs': tabs}
+        return {
+            "title": self.title % params,
+            "type": "tabcontainer",
+            "tabs": tabs,
+        }
 
 
 class Widget(JSONizable):
@@ -332,25 +454,19 @@ class ContentWidget(Widget, DataSource, AuthHandler):
     This widget acts as both a Widget and DataSource, since the Data used is
     simplistic.
     """
+
     datasource_handler_cls = ContentHandler
 
     def initialize(self, datasource=None, params=None):
         self.params = params
 
-
     @classmethod
     def to_json(cls):
-        """
-        to_json is an hybridmethod, the goal is to provide two different
-        implementations when it is used as a class (DataSource) or as
-        an instance (Widget).
-        """
         return {
-            'data_url': cls.data_url,
-            'name': cls.__name__,
-            'type': 'content',
-            'content': cls.__name__,
-            'title': cls.title
+            "data_url": cls.data_url,
+            "name": cls.__name__,
+            "type": cls.__name__,
+            "title": cls.title,
         }
 
     @classmethod
@@ -358,7 +474,6 @@ class ContentWidget(Widget, DataSource, AuthHandler):
         return cls.to_json()
 
 
-
 class Grid(Widget):
     """
     A rich table Widget, backed by a BackGrid component.
@@ -387,14 +502,15 @@ class Grid(Widget):
             if any(m._group != mg1 for m in self.metrics):
                 raise ValueError(
                     "A grid is not allowed to have metrics from different "
-                    "groups. (title: %s)" % self.title)
+                    "groups. (title: %s)" % self.title
+                )
 
     def to_json(self):
         values = self.__dict__.copy()
-        values['metrics'] = []
-        values['type'] = 'grid'
+        values["metrics"] = []
+        values["type"] = "grid"
         for metric in self.metrics:
-            values['metrics'].append(metric._fqn())
+            values["metrics"].append(metric._fqn())
         return values
 
 
@@ -403,10 +519,15 @@ class Graph(Widget):
     A widget backed by a Rickshaw graph.
     """
 
-    def __init__(self, title, grouper=None, url=None,
-                 axistype="time",
-                 metrics=None,
-                 **kwargs):
+    def __init__(
+        self,
+        title,
+        grouper=None,
+        url=None,
+        axistype="time",
+        metrics=None,
+        **kwargs,
+    ):
         self.title = title
         self.url = url
         self.grouper = grouper
@@ -426,14 +547,15 @@ class Graph(Widget):
         for metric in metrics:
             if metric.axis_type != axis_type:
                 raise ValueError(
-                    "Some metrics do not have the same x-axis type!")
+                    "Some metrics do not have the same x-axis type!"
+                )
 
     def to_json(self):
         values = self.__dict__.copy()
-        values['metrics'] = []
-        values['type'] = 'graph'
+        values["metrics"] = []
+        values["type"] = "graph"
         for metric in self.metrics:
-            values['metrics'].append(metric._fqn())
+            values["metrics"].append(metric._fqn())
         return values
 
 
@@ -446,7 +568,7 @@ class Declarative(object):
         self.args = args
         self.kwargs = kwargs
         global GLOBAL_COUNTER
-        self.kwargs.setdefault('_order', GLOBAL_COUNTER)
+        self.kwargs.setdefault("_order", GLOBAL_COUNTER)
         GLOBAL_COUNTER += 1
 
 
@@ -454,6 +576,7 @@ class MetricDef(Declarative):
     """
     A metric definition.
     """
+
     _cls = Metric
 
 
@@ -465,28 +588,26 @@ class MetaMetricGroup(type, JSONizable):
     """
 
     def __new__(meta, name, bases, dct):
-        dct['metrics'] = {}
-        dct['_stubs'] = {}
-        if not isinstance(dct.get('name', ''), str):
+        dct["metrics"] = {}
+        dct["_stubs"] = {}
+        if not isinstance(dct.get("name", ""), str):
             raise ValueError("The metric group name must be of type str")
         for base in bases:
             if hasattr(base, "enabled"):
                 dct.setdefault("enabled", base.enabled)
-            if hasattr(base, '_stubs'):
+            if hasattr(base, "_stubs"):
                 for key, stub in base._stubs.items():
-                    dct[key] = stub.__class__(*stub.args,
-                                              **stub.kwargs)
+                    dct[key] = stub.__class__(*stub.args, **stub.kwargs)
         for key, val in list(dct.items()):
             if isinstance(val, Declarative):
-                dct['_stubs'][key] = val
-                val.kwargs['name'] = key
+                dct["_stubs"][key] = val
+                val.kwargs["name"] = key
                 dct[key] = val = val._cls(*val.args, **val.kwargs)
             if isinstance(val, Metric):
                 dct.pop(key)
-                dct['metrics'][key] = val
+                dct["metrics"][key] = val
         return super(MetaMetricGroup, meta).__new__(meta, name, bases, dct)
 
-
     def __init__(cls, name, bases, dct):
         for metric in dct.get("metrics").values():
             metric.bind(cls)
@@ -501,26 +622,29 @@ class MetaMetricGroup(type, JSONizable):
         return key in cls.metrics
 
 
-
-
 class MetricGroupDef(with_metaclass(MetaMetricGroup, DataSource)):
     """
     Base class for MetricGroupDef.
 
     A MetricGroupDef provides syntactic sugar for instantiating MetricGroups.
     """
+
     _inst = None
-    metrics = {}
     datasource_handler_cls = MetricGroupHandler
 
     @classmethod
     def to_json(cls):
-        values = dict(((key, val) for key, val in cls.__dict__.items()
-                       if not key.startswith("_") and not isfunction(val)))
-        values['type'] = 'metric_group'
+        values = dict(
+            (
+                (key, val)
+                for key, val in cls.__dict__.items()
+                if not key.startswith("_") and not isfunction(val)
+            )
+        )
+        values["type"] = "metric_group"
         values.setdefault("xaxis", "ts")
-        values['metrics'] = list(cls.metrics.values())
-        values.pop("query")
+        values["metrics"] = list(cls.metrics.values())
+        values.pop("query", None)
         return values
 
     @classmethod
@@ -538,8 +662,43 @@ class MetricGroupDef(with_metaclass(MetaMetricGroup, DataSource)):
         if handler is None:
             return sorted(cls.metrics.values(), key=attrgetter("_order"))
 
-        return sorted(cls._get_metrics(handler, **params).values(),
-                      key=attrgetter("_order"))
+        return sorted(
+            cls._get_metrics(handler, **params).values(),
+            key=attrgetter("_order"),
+        )
+
+    @classmethod
+    def split(cls, handler, splits):
+        """
+        Returns a list of list containing all the metrics, split according
+        to the given list of list of metric names.  The array first element
+        contains all the metrics that don't appear in the splits.  For
+        instance:
+            split_metrics(..., [ ["b", "c"], ["d"] ])
+        called on a dict of metrics containing the key "a" to "e" would return
+        [[metric_a, metric_e], [metric_b, metric_c], [metric_d]]
+        """
+        metrics = cls._get_metrics(handler).copy()
+
+        res = []
+
+        i = 0
+        res.append({})
+
+        # Distribute the metrics according to the splits
+        for split in splits:
+            i += 1
+            res.append({})
+            for k in split:
+                res[i][k] = metrics.pop(k)
+
+        # and add what's left in the first position
+        res[0] = metrics
+
+        for i in range(0, len(res)):
+            res[i] = sorted(res[i].values(), key=attrgetter("_order"))
+
+        return res
 
     @property
     def metrics(self):
@@ -570,7 +729,10 @@ class DashboardPage(object):
     datasources = []
     parent = None
     timeline = None
-    docs_stats_url = 'https://powa.readthedocs.io/en/latest/components/stats_extensions/'
+    timeline_params = None
+    docs_stats_url = (
+        "https://powa.readthedocs.io/en/latest/components/stats_extensions/"
+    )
 
     @classmethod
     def url_specs(cls, url_prefix):
@@ -581,21 +743,32 @@ class DashboardPage(object):
         """
 
         url_specs = []
-        url_specs.append(URLSpec(
-            r"%s%s/" % (url_prefix, cls.base_url.strip("/")),
-            type(cls.__name__, (cls.dashboard_handler_cls, cls), {}), {
-                "template": cls.template,
-                "params": cls.params},
-            name=cls.__name__))
+        url_specs.append(
+            URLSpec(
+                r"%s%s/" % (url_prefix, cls.base_url.strip("/")),
+                type(cls.__name__, (cls.dashboard_handler_cls, cls), {}),
+                {"template": cls.template, "params": cls.params},
+                name=cls.__name__,
+            )
+        )
         for datasource in cls.datasources:
             if datasource.data_url is None:
-                raise KeyError("A Datasource must have a data_url: %s" %
-                               datasource.__name__)
-            url_specs.append(URLSpec(
-                r"%s%s/" % (url_prefix, datasource.data_url.strip("/")),
-                type(datasource.__name__, (datasource, datasource.datasource_handler_cls),
-                     dict(datasource.__dict__)),
-                {"datasource": datasource, "params": cls.params}, name=datasource.url_name))
+                raise KeyError(
+                    "A Datasource must have a data_url: %s"
+                    % datasource.__name__
+                )
+            url_specs.append(
+                URLSpec(
+                    r"%s%s/" % (url_prefix, datasource.data_url.strip("/")),
+                    type(
+                        datasource.__name__,
+                        (datasource, datasource.datasource_handler_cls),
+                        dict(datasource.__dict__),
+                    ),
+                    {"datasource": datasource, "params": cls.params},
+                    name=datasource.url_name,
+                )
+            )
         return url_specs
 
     @classmethod
@@ -604,17 +777,19 @@ class DashboardPage(object):
 
     @classmethod
     def get_selfmenu(cls, handler, params):
-        my_params = OrderedDict((key, params.get(key))
-                                for key in cls.params)
+        my_params = OrderedDict((key, params.get(key)) for key in cls.params)
         return MenuEntry(cls.title % params, cls.__name__, my_params)
 
     @classmethod
     def get_breadcrumb(cls, handler, params):
-        if (getattr(cls, "breadcrum_title", None)):
+        if getattr(cls, "breadcrum_title", None):
             title = cls.breadcrum_title(handler, params)
         else:
             title = cls.title % params
         entry = MenuEntry(title, cls.__name__, params)
+        entry.children_title = (
+            cls.title % params if hasattr(cls, "title") else None
+        )
         entry.children = cls.get_childmenu(handler, params)
         items = [entry]
 
@@ -624,8 +799,10 @@ class DashboardPage(object):
         else:
             parent_params = []
 
-        if cls.parent is not None and hasattr(handler, 'parent') and \
-            hasattr(cls.parent, 'get_breadcrumb'):
-            items.extend(cls.parent.get_breadcrumb(handler,
-                                                   parent_params))
+        if (
+            cls.parent is not None
+            and hasattr(handler, "parent")
+            and hasattr(cls.parent, "get_breadcrumb")
+        ):
+            items.extend(cls.parent.get_breadcrumb(handler, parent_params))
         return items
